@@ -1,7 +1,7 @@
 /*
    rdesktop: A Remote Desktop Protocol client.
    Miscellaneous protocol constants
-   Copyright (C) Matthew Chapman 1999-2000
+   Copyright (C) Matthew Chapman 1999-2002
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,11 +24,11 @@
 /* ISO PDU codes */
 enum ISO_PDU_CODE
 {
-	ISO_PDU_CR = 0xE0, /* Connection Request */
-	ISO_PDU_CC = 0xD0, /* Connection Confirm */
-	ISO_PDU_DR = 0x80, /* Disconnect Request */
-	ISO_PDU_DT = 0xF0, /* Data */
-	ISO_PDU_ER = 0x70  /* Error */
+	ISO_PDU_CR = 0xE0,	/* Connection Request */
+	ISO_PDU_CC = 0xD0,	/* Connection Confirm */
+	ISO_PDU_DR = 0x80,	/* Disconnect Request */
+	ISO_PDU_DT = 0xF0,	/* Data */
+	ISO_PDU_ER = 0x70	/* Error */
 };
 
 /* MCS PDU codes */
@@ -54,6 +54,7 @@ enum MCS_PDU_TYPE
 #define MCS_TAG_DOMAIN_PARAMS	0x30
 
 #define MCS_GLOBAL_CHANNEL	1003
+#define MCS_USERCHANNEL_BASE    1001
 
 /* RDP secure transport constants */
 #define SEC_RANDOM_SIZE		32
@@ -68,27 +69,31 @@ enum MCS_PDU_TYPE
 
 #define SEC_TAG_SRV_INFO	0x0c01
 #define SEC_TAG_SRV_CRYPT	0x0c02
-#define SEC_TAG_SRV_3		0x0c03
+#define SEC_TAG_SRV_CHANNELS	0x0c03
 
 #define SEC_TAG_CLI_INFO	0xc001
 #define SEC_TAG_CLI_CRYPT	0xc002
+#define SEC_TAG_CLI_CHANNELS    0xc003
+#define SEC_TAG_CLI_4           0xc004
 
 #define SEC_TAG_PUBKEY		0x0006
 #define SEC_TAG_KEYSIG		0x0008
 
-#define SEC_RSA_MAGIC		0x31415352 /* RSA1 */
+#define SEC_RSA_MAGIC		0x31415352	/* RSA1 */
 
 /* RDP licensing constants */
 #define LICENCE_TOKEN_SIZE	10
 #define LICENCE_HWID_SIZE	20
 #define LICENCE_SIGNATURE_SIZE	16
 
-#define LICENCE_TAG_DEMAND	0x0201
-#define LICENCE_TAG_AUTHREQ	0x0202
-#define LICENCE_TAG_ISSUE	0x0203
-#define LICENCE_TAG_REQUEST	0x0213
-#define LICENCE_TAG_AUTHRESP	0x0215
-#define LICENCE_TAG_RESULT	0x02ff
+#define LICENCE_TAG_DEMAND	0x01
+#define LICENCE_TAG_AUTHREQ	0x02
+#define LICENCE_TAG_ISSUE	0x03
+#define LICENCE_TAG_REISSUE	0x04
+#define LICENCE_TAG_PRESENT	0x12
+#define LICENCE_TAG_REQUEST	0x13
+#define LICENCE_TAG_AUTHRESP	0x15
+#define LICENCE_TAG_RESULT	0xff
 
 #define LICENCE_TAG_USER	0x000f
 #define LICENCE_TAG_HOST	0x0010
@@ -111,7 +116,8 @@ enum RDP_DATA_PDU_TYPE
 	RDP_DATA_PDU_SYNCHRONISE = 31,
 	RDP_DATA_PDU_BELL = 34,
 	RDP_DATA_PDU_LOGON = 38,
-	RDP_DATA_PDU_FONT2 = 39
+	RDP_DATA_PDU_FONT2 = 39,
+	RDP_DATA_PDU_DISCONNECT = 47
 };
 
 enum RDP_CONTROL_PDU_TYPE
@@ -132,11 +138,18 @@ enum RDP_UPDATE_PDU_TYPE
 
 enum RDP_POINTER_PDU_TYPE
 {
+	RDP_POINTER_SYSTEM = 1,
 	RDP_POINTER_MOVE = 3,
 	RDP_POINTER_COLOR = 6,
 	RDP_POINTER_CACHED = 7
 };
 
+enum RDP_SYSTEM_POINTER_TYPE
+{
+	RDP_NULL_POINTER = 0,
+	RDP_DEFAULT_POINTER = 0x7F00
+};
+
 enum RDP_INPUT_DEVICE
 {
 	RDP_INPUT_SYNCHRONIZE = 0,
@@ -153,10 +166,21 @@ enum RDP_INPUT_DEVICE
 #define KBD_FLAG_DOWN           0x4000
 #define KBD_FLAG_UP             0x8000
 
+/* These are for synchronization; not for keystrokes */
+#define KBD_FLAG_SCROLL   0x0001
+#define KBD_FLAG_NUMLOCK  0x0002
+#define KBD_FLAG_CAPITAL  0x0004
+
+/* See T.128 */
+#define RDP_KEYPRESS 0
+#define RDP_KEYRELEASE (KBD_FLAG_DOWN | KBD_FLAG_UP)
+
 #define MOUSE_FLAG_MOVE         0x0800
 #define MOUSE_FLAG_BUTTON1      0x1000
 #define MOUSE_FLAG_BUTTON2      0x2000
 #define MOUSE_FLAG_BUTTON3      0x4000
+#define MOUSE_FLAG_BUTTON4      0x0280
+#define MOUSE_FLAG_BUTTON5      0x0380
 #define MOUSE_FLAG_DOWN         0x8000
 
 /* Raster operation masks */
@@ -176,7 +200,7 @@ enum RDP_INPUT_DEVICE
 #define TEXT2_IMPLICIT_X	0x20
 
 /* RDP capabilities */
-#define RDP_CAPSET_GENERAL	1
+#define RDP_CAPSET_GENERAL	1	/* Maps to generalCapabilitySet in T.128 page 138 */
 #define RDP_CAPLEN_GENERAL	0x18
 #define OS_MAJOR_TYPE_UNIX	4
 #define OS_MINOR_TYPE_XSERVER	7
@@ -215,3 +239,81 @@ enum RDP_INPUT_DEVICE
 /* Logon flags */
 #define RDP_LOGON_NORMAL	0x33
 #define RDP_LOGON_AUTO		0x8
+#define RDP_LOGON_BLOB		0x100
+
+/* Keymap flags */
+#define MapRightShiftMask   (1<<0)
+#define MapLeftShiftMask    (1<<1)
+#define MapShiftMask (MapRightShiftMask | MapLeftShiftMask)
+
+#define MapRightAltMask     (1<<2)
+#define MapLeftAltMask      (1<<3)
+#define MapAltGrMask MapRightAltMask
+
+#define MapRightCtrlMask    (1<<4)
+#define MapLeftCtrlMask     (1<<5)
+#define MapCtrlMask (MapRightCtrlMask | MapLeftCtrlMask)
+
+#define MapRightWinMask     (1<<6)
+#define MapLeftWinMask      (1<<7)
+#define MapWinMask (MapRightWinMask | MapLeftWinMask)
+
+#define MapNumLockMask      (1<<8)
+#define MapCapsLockMask     (1<<9)
+
+#define MapLocalStateMask   (1<<10)
+
+#define MapInhibitMask      (1<<11)
+
+#define MASK_ADD_BITS(var, mask) (var |= mask)
+#define MASK_REMOVE_BITS(var, mask) (var &= ~mask)
+#define MASK_HAS_BITS(var, mask) ((var & mask)>0)
+#define MASK_CHANGE_BIT(var, mask, active) (var = ((var & ~mask) | (active ? mask : 0)))
+
+/* Clipboard constants, "borrowed" from GCC system headers in 
+   the w32 cross compiler */
+
+#define CF_TEXT         1
+#define CF_BITMAP       2
+#define CF_METAFILEPICT 3
+#define CF_SYLK         4
+#define CF_DIF          5
+#define CF_TIFF         6
+#define CF_OEMTEXT      7
+#define CF_DIB          8
+#define CF_PALETTE      9
+#define CF_PENDATA      10
+#define CF_RIFF         11
+#define CF_WAVE         12
+#define CF_UNICODETEXT  13
+#define CF_ENHMETAFILE  14
+#define CF_HDROP        15
+#define CF_LOCALE       16
+#define CF_MAX          17
+#define CF_OWNERDISPLAY 128
+#define CF_DSPTEXT      129
+#define CF_DSPBITMAP    130
+#define CF_DSPMETAFILEPICT      131
+#define CF_DSPENHMETAFILE       142
+#define CF_PRIVATEFIRST 512
+#define CF_PRIVATELAST  767
+#define CF_GDIOBJFIRST  768
+#define CF_GDIOBJLAST   1023
+
+/* Sound format constants */
+#define WAVE_FORMAT_PCM		1
+#define WAVE_FORMAT_ADPCM	2
+#define WAVE_FORMAT_ALAW	6
+#define WAVE_FORMAT_MULAW	7
+
+/* Virtual channel options */
+#define CHANNEL_OPTION_INITIALIZED	0x80000000
+#define CHANNEL_OPTION_ENCRYPT_RDP	0x40000000
+#define CHANNEL_OPTION_COMPRESS_RDP	0x00800000
+#define CHANNEL_OPTION_SHOW_PROTOCOL	0x00200000
+
+/* NT status codes for RDPDR */
+#define STATUS_SUCCESS			0x00000000
+#define STATUS_INVALID_PARAMETER	0xc000000d
+#define STATUS_INVALID_DEVICE_REQUEST	0xc0000010
+#define STATUS_ACCESS_DENIED		0xc0000022
