@@ -1,7 +1,7 @@
 /*
    rdesktop: A Remote Desktop Protocol client.
    Master include file
-   Copyright (C) Matthew Chapman 1999-2000
+   Copyright (C) Matthew Chapman 1999-2004
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#define VERSION "1.1.0"
+#define VERSION "1.3.1"
 
 #ifdef WITH_DEBUG
 #define DEBUG(args)	printf args;
@@ -30,11 +30,35 @@
 #define DEBUG(args)
 #endif
 
-#define STRNCPY(dst,src,n) { strncpy(dst,src,n-1); dst[n-1] = 0; }
+#ifdef WITH_DEBUG_KBD
+#define DEBUG_KBD(args) printf args;
+#else
+#define DEBUG_KBD(args)
+#endif
 
+#ifdef WITH_DEBUG_RDP5
+#define DEBUG_RDP5(args) printf args;
+#else
+#define DEBUG_RDP5(args)
+#endif
+
+#ifdef WITH_DEBUG_CLIPBOARD
+#define DEBUG_CLIPBOARD(args) printf args;
+#else
+#define DEBUG_CLIPBOARD(args)
+#endif
+
+#define STRNCPY(dst,src,n)	{ strncpy(dst,src,n-1); dst[n-1] = 0; }
+#ifndef MIN
+#define MIN(x,y)		(((x) < (y)) ? (x) : (y))
+#endif
+#ifndef MAX
+#define MAX(x,y)		(((x) > (y)) ? (x) : (y))
+#endif
+
+#include "parse.h"
 #include "constants.h"
 #include "types.h"
-#include "parse.h"
 
 #ifndef MAKE_PROTO
 #include "proto.h"
